#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <string.h>

#include "non_gui_defs.h"
#include "gui_defs.h"
#include "startup.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"

/* start a new game: create dogs and kennels, set default options etc.
   if new_game is 1, we start a new game from within a game */
gint start(GtkWidget *widget, gint new_game)
{
    GtkWidget *main_window = lookup_widget(widget, "main_window");
    GtkWidget *ask_name_window = create_ask_name_window();
    gint i;

    /* set default save file to NULL so that the first time
       'save' is clicked the file selection window pops up */
    game_file = NULL;

    /* generate dogs */
    fill_in_kennels(names_kept);

    if(new_game == 1)
	free_histories();
    /* clearly there is no history yet */
    first_history = NULL;
    
    /* write the first races */
    write_races();

    if(new_game == 0)
	set_default_options();
    
    /* nullify all-time records
       and 'qualified_kennels' */
    set_default_records();

    /* set orders */
    for(i=0;i<420;i++)
    {
	if(i < 400)
	{
	    statistics_order[0][i] = i;
	    statistics_order[1][i] = i;
	}
	else
	{
	    statistics_order[0][i] = i - 400;
	    statistics_order[1][i] = i - 400;
	}
	
	order[i] = i;
    }

    season = month = week = 1;

    /* show the main scren with the home kennel */
    display_kennel(widget, 1, 0, 0);

    /* set sensitivity of all buttons,
       write appropriate labels */
    write_labels(main_window);
    set_buttons(widget);

    if(new_game == 0)
	gtk_widget_show(ask_name_window);
    else
	gtk_widget_show(main_window);
    
    return 0;
}


/* fill in the 19 cpu-kennels and the human player's
   kennel (which is the first one in the kennel
   array).

   the dog array is also filled in in this function although it's kept
   separately from the kennel structs. however, a dog's id always determines
   non-ambiguously its kennel.

   we keep 500 names from the name file in the memory to have
   funky names for pups when breeding */
gint fill_in_kennels(gchar names_kept[][19])
{

    GtkWidget *progress_window = create_progress_window();
    GtkWidget *progress_bar = lookup_widget(progress_window, 
					    "progress_window_progress_bar");
    gint i;
    gint random;
    gint kennel_names[19];
    gint dog_names[500];
    FILE *fil;
    gchar names[8424][19];
    gchar trash[2];

    gtk_widget_show(progress_window);

    /* we use kennel_names for keeping track of the names
       already assigned, so as not to have two kennels with 
       identical names */
    for(i=0;i<500;i++)
    {
	dog_names[i] = -1;
	if(i < 19)
	    kennel_names[i] = -1;
    }

    /* read dog names and kennel names into a buffer
       from which they are chosen randomly afterwards.
       the last 200 names are kennel names, the rest dog
       names */
    if((fil=fopen(my_find_pixmap_file("names"),"r")) == NULL)
    {
	printf("Could not open 'names' file.\n");
	exit(1);
    }
    for(i=0;i<8424;i++)
    {
	fscanf(fil,"%[^\n]",names[i]);
	fscanf(fil,"%[\n]",trash);
    }
    fclose(fil);
    
    /* copy 500 random names into the gchar array we'll keep */
    i=500;
    while(i > 0)
    {
	random = (gint)rint( rnd(0,8223) );
	while(is_in_list(dog_names, 0, 499, random) == 1)
	    random = (gint)rint( rnd(0,8223) );
	i--;
	strcpy(names_kept[i],names[random]);
	dog_names[i] = random;
    }

    /* set puppies to 'invisible' since at the beginning
     of the game the human player has no puppies yet */
    for(i=400;i<420;i++)
	dogs[i].age = -1;

    for(i=0;i<20;i++)
    {
	/* for cpu-kennels copy a random name from the buffer to
	   'Kennel.name' */
	if(i > 0)
	{
	    random = (gint)rint( rnd(8224,8423) );
	    while(is_in_list(kennel_names, 0, 18, random) == 1)
		random = (gint)rint( rnd(8224,8423) );
	    strcpy( kennels[i].name, names[random] );
	    kennel_names[i - 1] = random;
	}
    }
    
    /* generate the dogs for each kennel.
       each one consists of 20 dogs except the first one which is
       the human player's kennel. it consists of only 15 dogs
       to leave room for breeding.
       the dog with index (or id) n belongs to the kennel with index
       (or id) n / 20 rounded off.
       
       we generate the dogs in batches of 20 to have roughly a similar
       distribution of dogs on different levels in each kennel */

    /* the dog ids we write here because generate_batch
       only fills in the real dogs and not the invisible ones */
    for(i=0;i<420;i++)
	dogs[i].id = i;
        
    for(i=0;i<20;i++)
    {
	gtk_progress_bar_set_fraction(
	    GTK_PROGRESS_BAR(progress_bar), (gfloat)i / 19);

	while(gtk_events_pending())
	    gtk_main_iteration();
	
	generate_batch(names, i);
    }
    
    gtk_widget_destroy(progress_window);
    
    return 0;
}

/* set the options variables to default value */
gint set_default_options(void)
{
    gint i;

    for(i=0;i<19;i++)
    {
	options_kennel_block[0][i] = 0;
	options_kennel_block[1][i] = 0;
    }

    /* confirm everything, show each news */
    for(i=0;i<12;i++)
    {
	options_confirm[i] = 1;
	options_show_news[i] = 1;
    }

    for(i=0;i<50;i++)
	options_misc[i] = 1;
    
    /* no notification about bitches in heat */
    options_misc[18] = options_misc[19] = 199;

    /* live race duration factor */
    options_misc[0] = 0;

    return 0;
}


gint set_default_records(void)
{
    gint i;

    for(i=0;i<7;i++)
    {
	strcpy(dog_records[i].name, "NAME");
	strcpy(dog_records[i].kennel_name, "NAME");
	dog_records[i].season = 0;
	if(i < 3)
	    dog_records[i].value = 100;
	else
	    dog_records[i].value = 0;

	if(i<5)
	{
	    strcpy(kennel_records[i].name, "NAME");
	    kennel_records[i].season = 0;
	    if(i != 0)
		kennel_records[i].value = 0;
	    else
		kennel_records[i].value = 100;
	}
	if(i<6)
	    qualified_kennels[i] = i;
    }

    return 0;
}
